#include <i86.h>
#include <stdlib.h>
#include <sys/irqinfo.h>
#include <sys/seginfo.h>
//#include <sys/osinfo.h>
//#include <sys/pci.h>
#include "struct.h"
#include "extern.h"
#include "proto.h"

/****************************************************************************/
/*Function hardware_init()                                                  */
/*              This function is called from the main() during the          */
/*              initialization of the driver.  This function will           */
/*              preform any hardware specific operations before any         */
/*              open/close/read/write requests are processed.               */
/*     Parameters Passed: none                                              */
/****************************************************************************/

int TMK_TuneIODelay(unsigned nMaxIODelay);

void hardware_init()
{
  int iRt, iRt1;
  
  if (tmkconfig(0, nTmkType, wPort1, wPort2, nIrq1, nIrq2))
  {
    perror("tmk1553b:hw_init:tmkconfig error");
    exit(-1);
  }

  hTmk = 0;
  aTmkConfig[hTmk].nType = (unsigned short)nTmkType;
  if (nTmkType == TAI)
    aTmkConfig[hTmk].wPorts1 = __tmkIrqPort[hTmk] - 0x6;
  else if (nTmkType == TMKXI || nTmkType == MRTXI)
  {
    if (__tmkIrqBit[hTmk] & 0x8000)
    {
      aTmkConfig[hTmk].fLocalReadInt = 1;
      aTmkConfig[hTmk].wPorts1 = __tmkIrqPort[hTmk] - 0xA;
    }
    else
    {
      aTmkConfig[hTmk].fLocalReadInt = 0;
      aTmkConfig[hTmk].wHiddenPorts = __tmkIrqPort[hTmk] - 0x4C;
    }
  }
/*
  aTmkConfig[hTmk].wPorts1 = (unsigned short)wPort1;
  aTmkConfig[hTmk].wPorts2 = (unsigned short)wPort2;
//  aTmkConfig[hTmk].wIrq1 = (unsigned short)nIrq1;
//  aTmkConfig[hTmk].wIrq2 = (unsigned short)nIrq2;
//  aTmkConfig[hTmk].nLoadType = tmkLoadType;
  aTmkConfig[hTmk].wHiddenPorts = (unsigned short)wPort2;
//  aTmkConfig[hTmk].nBusType = tmkBusType;
//  aTmkConfig[hTmk].nBusNumber = tmkBusNumber;
//  aTmkConfig[hTmk].nDev = tmkDev;
//  aTmkConfig[hTmk].nSubDev = tmkSubDev;
//  if (tmkBusType == PCIBus)
//    aTmkConfig[hTmk].fIrqShared = 1;
  aTmkConfig[hTmk].fLocalReadInt = tmkLocalReadInt;
*/
  aTmkConfig[hTmk].fMRT = 0;

  switch (nTmkType)
  {
  case TMK400:
  case TMKMPC:
  case RTMK400:
    tmkiodelay(30);
  case TMKX:
  case TMKXI:
  case TA:
  case TAI:
    if (TMK_TuneIODelay(0))
    {
      perror("tmk1553b:hw_init:I/O (RAM) test error");
      if (!fIgnoreTestError)
        exit(-1);
    }
    bcreset();
    break;
//#ifdef LOADMR6
  case MRTX:
  case MRTXI:
    bcreset();
    iRt = mrtgetrt0();
    aTmkConfig[hTmk].fMRT = 0;
    iRt1 = iRt + mrtgetnrt();
    #ifdef MY_DBG
    KdPrint(("Tmk1553b: MRT:"));
    #endif
    while (iRt < iRt1)
    {
      tmkError = 0;
      tmkselect(iRt);
      rtreset();
      #ifdef MY_DBG
      if (tmkError)
        KdPrint(("-"));
      else
        KdPrint(("+"));
      #endif
      bcreset(); 
      tmkError = 0;
      ++iRt;
    }
    tmkselect(0);
    #ifdef MY_DBG
    KdPrint(("\n"));
    #endif
    break;
//#endif //def LOADMR6
#ifdef LOADTT6
  case TTX:
  case TTXI:
#endif //def LOADTT6
#ifdef LOADTT6
  case TDX:
  case TDXI:
#endif //def LOADTD6
    break;
  }
}

/****************************************************************************/
/*      TmkTest v2.01. ELCUS, 1995,1998.                                    */
/****************************************************************************/

#include <stdlib.h>

#define TMK_RAM_TEST_BLK
#define TMK_DEFAULT_MAX_IO_DELAY 50

#define BC_RAM_ERROR 40
#define RT_RAM_ERROR 41

#ifndef RT_GOOD
#define RT_GOOD 0
#endif
#ifndef RT_BAD_RAM
#define RT_BAD_RAM 1
#endif

#ifdef TMK_RAM_TEST_BLK

unsigned short TMK_RAMTestBuf[64];

int BC_RAMTest()
{
 unsigned wMaxBase;
 int nResult;
 register unsigned wBase;
 register unsigned wAddr;

 wMaxBase = bcgetmaxbase(); 
 srand(1);
 for (wBase = 0; wBase <= wMaxBase; wBase++)
 {
  for (wAddr = 0; wAddr <= 63; wAddr++)
   TMK_RAMTestBuf[wAddr] = (unsigned short)((unsigned)rand() + (unsigned)rand());
  bcdefbase(wBase);
  bcputblk(0, TMK_RAMTestBuf, 64);
 }
 nResult = 0;
 srand(1);
 for (wBase = 0; !nResult && wBase <= wMaxBase; wBase++)
 {
  bcdefbase(wBase);
  bcgetblk(0, TMK_RAMTestBuf, 64);
  for (wAddr = 0; wAddr <= 63; wAddr++)
   if (TMK_RAMTestBuf[wAddr] != (unsigned short)((unsigned)rand() + (unsigned)rand()))
   {
    nResult = BC_RAM_ERROR;
    break;
   }
 }
 return nResult;
}

#else

int BC_RAMTest()
{
 unsigned wMaxBase;
 int nResult;
 register unsigned wBase;
 register unsigned wAddr;

 wMaxBase = bcgetmaxbase();
 srand(1);
 for (wBase = 0; wBase <= wMaxBase; wBase++)
 {
  bcdefbase(wBase);
  for (wAddr = 0; wAddr <= 63; wAddr++)
   bcputw(wAddr, (unsigned)rand() + (unsigned)rand());
 }
 nResult = 0;
 srand(1);
 for (wBase = 0; !nResult && wBase <= wMaxBase; wBase++)
 {
  bcdefbase(wBase);
  for (wAddr = 0; wAddr <= 63; wAddr++)
   if (bcgetw(wAddr) != (unsigned short)((unsigned)rand() + (unsigned)rand()))
   {
    nResult = BC_RAM_ERROR;
    break;
   }
 }
 return nResult;
}

#endif /*def TMK_RAM_TEST_BLOCK*/

int RT_RAMTest()
{
 unsigned wMaxPage, wPage;
 int nResult;
 register unsigned wSubAddr;
 register unsigned wAddr;

 wMaxPage = rtgetmaxpage();
 srand(1);
 for (wPage = 0; wPage <= wMaxPage; wPage++)
 {
  rtdefpage(wPage);
  for (wSubAddr = 0; wSubAddr <= 31; wSubAddr++)
  {
   rtdefsubaddr(RT_RECEIVE, wSubAddr);
   for (wAddr = 0; wAddr <= 31; wAddr++)
    rtputw(wAddr, (unsigned)rand() + (unsigned)rand());
   rtdefsubaddr(RT_TRANSMIT, wSubAddr);
   for (wAddr = 0; wAddr <= 31; wAddr++)
    rtputw(wAddr, (unsigned)rand() + (unsigned)rand());
  }
 }
 nResult = 0;
 srand(1);
 for (wPage = 0; !nResult && wPage <= wMaxPage; wPage++)
 {
  rtdefpage(wPage);
  for (wSubAddr = 0; wSubAddr <= 31; wSubAddr++)
  {
   rtdefsubaddr(RT_RECEIVE, wSubAddr);
   for (wAddr = 0; wAddr <= 31; wAddr++)
    if (rtgetw(wAddr) != (unsigned short)((unsigned)rand() + (unsigned)rand()))
    {
     nResult = RT_RAM_ERROR;
     break;
    }
   rtdefsubaddr(RT_TRANSMIT, wSubAddr);
   for (wAddr = 0; wAddr <= 31; wAddr++)
    if (rtgetw(wAddr) != (unsigned short)((unsigned)rand() + (unsigned)rand()))
    {
     nResult = RT_RAM_ERROR;
     break;
    }
  }
 }
 return nResult;
}

int RT_Test()
{
 int nResult;

 rtsetanswbits(BUSY);
 if ((nResult = RT_RAMTest()) != 0)
 {
  rtputcmddata(0x413, RT_BAD_RAM);
  rtsetanswbits(RTFL);
 }
 else
 {
  rtputcmddata(0x413, RT_GOOD);
  rtclranswbits(RTFL);
 }
 rtclranswbits(BUSY);
 return nResult;
}

int TMK_TuneIODelay(unsigned nMaxIODelay)
{
 int fLastError, fErrors;
 unsigned nIODelay, nSaveIODelay;

 if (nMaxIODelay == 0)
  nMaxIODelay = TMK_DEFAULT_MAX_IO_DELAY;
 nIODelay = nSaveIODelay = tmkiodelay(GET_IO_DELAY);
 fErrors = 0;
 fLastError = 0;
 do
 {
  tmkiodelay(nIODelay);
  bcreset();
  if ((fLastError = BC_RAMTest()) != 0)
  {
   fErrors = 1;
   nIODelay++;
  }
  else
  {
   if (!fErrors)
    nIODelay--;
   else
    break;
  }
 }
 while (nIODelay > 0 && nIODelay <= nMaxIODelay);
 if (fLastError)
  tmkiodelay(nSaveIODelay);
 else
  tmkiodelay(nIODelay+1);
 return fLastError;
}
