#include <stdlib.h>
#include <sys/fd.h>
#include "define.h"
#include "struct.h"
#include "extern.h"

/****************************************************************************/
/*Function close_msg()                                                      */
/*              This function is called when a IO_CLOSE message is          */
/*              received.  This message is sent every time a                */
/*              application issues a close() on the file                    */
/*              descriptor associated with the driver.  This                */
/*              function will reset the adapter and allow another           */
/*              application to open the driver.                             */
/*     Parameters Passed :                                                  */
/*      pid :Process ID of application calling close().                     */
/****************************************************************************/
int close_msg(pid_t pid)
{
  struct ocb *ocb;
   
  if ((ocb = get_ocb(pid, io_msg.close.fd)) == -1)
  {
    io_msg.status = EBADF;
    return (REPLY);
  }
  ocb->count--;
  if (!(ocb->count))   /* Another fd is using this OCB. */
  {
    for (pOCB = &FirstOCB; pOCB != NULL; pOCB = pOCB->pNext)
    {
      if (pOCB->pNext == ocb)
      {
        pOCB->pNext = ocb->pNext;
        break;
      }
    }
    switch (wTmkMode)
    {
    case BC_MODE:
      if (pTmkLockOCB == ocb)
      {
        bcreset();
        pTmkLockOCB = NULL;
        pTmkStartOCB = NULL;
      }
      if (pRtArbLockOCB == ocb)
        pRtArbLockOCB = NULL;
      if (pRtAnswLockOCB == ocb)
        pRtAnswLockOCB = NULL;
      if (!(wTmkModeDeep -= ocb->wModeDeep))
        wTmkMode = UNDEFINED_MODE;
      break;
    case RT_MODE:
      if (pTmkLockOCB == ocb)
      {
        rtreset();
        pTmkLockOCB = NULL;
      }
      if (pRtArbLockOCB == ocb)
      {
        pRtArbLockOCB = NULL;
        rtunlock();
//        rtdefsubaddr(wDir, wSubAddr);
//        rtunlock(wLockDir,wLockSubAddr);
      }   
      if (pRtAnswLockOCB == ocb)
        pRtAnswLockOCB = NULL;
      if (!(wTmkModeDeep -= ocb->wModeDeep))
        wTmkMode = UNDEFINED_MODE;
      break;
    case MT_MODE:
      if (pTmkLockOCB == ocb)
      {
        mtreset();
        pTmkLockOCB = NULL;
        pTmkStartOCB = NULL;
      }
      if (pRtArbLockOCB == ocb)
        pRtArbLockOCB = NULL;
      if (pRtAnswLockOCB == ocb)
        pRtAnswLockOCB = NULL;
      if (!(wTmkModeDeep -= ocb->wModeDeep))
        wTmkMode = UNDEFINED_MODE;
      break;
    case MRT_MODE:
      if (pTmkLockOCB == ocb)
      {
        bcreset();
        pTmkLockOCB = NULL;
      }
      if (pRtArbLockOCB == ocb)
      {
        pRtArbLockOCB = NULL;
        rtunlock();
//        rtdefsubaddr(wDir, wSubAddr);
//        rtunlock(wLockDir,wLockSubAddr);
      }
      if (pRtAnswLockOCB == ocb)
        pRtAnswLockOCB = NULL;
      if (!(wTmkModeDeep -= ocb->wModeDeep))
        wTmkMode = UNDEFINED_MODE;
      break;
    case UNDEFINED_MODE:
      break;
    }
    free(ocb);
  }
      
  /* Have to map a zero where this ocb was. */
  map_ocb(pid, io_msg.close.fd, NULL);

  io_msg.close_reply.status = EOK;
  return (REPLY);
}
