#include <stdlib.h>
#include "struct.h"
#include "extern.h"

/****************************************************************************/
/*Function handle_msg()                                                     */
/*              This function is called when the IO_HANDLE message          */
/*              is called.                                                  */
/*     Parameters Passed:                                                   */
/*      pid :Process ID of calling application.                             */
/****************************************************************************/
int handle_msg(pid_t pid)
{
  uid_t uid;
  gid_t gid;
  struct ocb *ocb;

  get_ids(pid, &uid, &gid);       /* Get egid and euid of process pid.*/
  io_msg.status = EOK;

  switch (io_msg.open.oflag)
  {
    case _IO_HNDL_CHANGE:
      if((uid != 0) && (uid != tmk_stat.st_uid))
      {
        io_msg.status = EPERM;            /* Must be super user or have equivalent euid */
        break;
      }
           
      if ((ocb = (struct ocb *)calloc(1, sizeof(struct ocb))) == NULL)
      {
        io_msg.status = ENOMEM;
        return (REPLY);
      }
      
      if(map_ocb(pid, io_msg.open.fd, ocb) == -1)
        io_msg.status = errno;
      else
        ocb->count++;
      break;

    case _IO_HNDL_INFO:
    case _IO_HNDL_UTIME:
      if((check_access(pid, &tmk_stat) & S_IRWXU) == 0)
      {
        io_msg.status = EPERM;
        break;
      }
      if ((ocb = (struct ocb *)calloc(1, sizeof(struct ocb))) == NULL)
      {
        io_msg.status = ENOMEM;
        return (REPLY);
      }
         
      if(map_ocb(pid, io_msg.open.fd, ocb) == -1)
        io_msg.status = errno;
      else
        ocb->count++;
      break;

    case _IO_HNDL_RDDIR:
    case _IO_HNDL_LOAD:
    case _IO_HNDL_CLOAD:
      io_msg.status = ENOSYS;
      break;
  }
  return (REPLY);
}

