#include <sys/fd.h>
#include <stdlib.h>
#include "define.h"
#include "struct.h"
#include "extern.h"

/****************************************************************************/
/*Function open_msg()                                                       */
/*              This function is called when the IO_OPEN message            */
/*              is received.  This function will check access               */
/*              rights of the application opening the driver as             */
/*              well as make sure the driver is not already open.           */
/*              It will then associate a OCB with the file                  */
/*              descriptor which is sent with the message.                  */
/*     Parameters Passed:                                                   */
/*      pid :Process ID of calling application.                             */
/****************************************************************************/
int open_msg(pid_t pid)
{
  mode_t mode;
  struct ocb *ocb;

  if (((mode = check_access(pid, &tmk_stat)) & S_IRWXU) == 0)
  {
    io_msg.status = EPERM;
    return (REPLY);
  }

  if ((ocb = (struct ocb *)calloc(1, sizeof(struct ocb))) == -1)
  {
    io_msg.status = EBADF;
    return (REPLY);
  }
   
  if (map_ocb(pid, io_msg.open.fd, ocb) == -1)
    return (REPLY);
    
  ocb->wTmkNumber = io_msg.open.unit;
  ocb->count++;
  ocb->mode = mode; /* Save access mode bits. */
  ocb->oflags = io_msg.open.oflag;
  io_msg.open_reply.status = EOK;
  return (REPLY);
}
