#include <stdlib.h>
#include <signal.h>
#include <i86.h>
#include <stddef.h>
#include <dirent.h>
#include <unistd.h>
#include <sys/psinfo.h>
#include <sys/osinfo.h>
#include <sys/sched.h>
#include <sys/prfx.h>
#include <sys/fd.h>
#include <sys/types.h>
#include <sys/proxy.h>
#include <sys/stat.h>
#include "struct.h"
#include "extern.h"

/****************************************************************************/
/*Function manager_init()                                                   */
/*              This function is called from the main() during the          */
/*              initialization of the driver.  This function will           */
/*              register this application as a device driver, notify the    */
/*              kernel that we are a server so thatwe will receive          */
/*              system messages.                                            */
/*     Parameters Passed:                                                   */
/*      char *devicename, int priority                                      */
/****************************************************************************/
void manager_init(int priority)
{
  int i;
  struct _osinfo osdata;

  setscheduler(0, SCHED_FIFO, priority);    /* Boost our priority.     */

   /* Allow only SIGTERM for graceful termination, ignore all others. */
  for (i = _SIGMIN; i <= _SIGMAX; ++i) /* Ignore signals all signal except SIGTERM. */
    if (i != SIGTERM)
      signal(i, SIG_IGN);

  signal(SIGTERM, &terminate);  /* Setup the terminate signal handler function. */

  if (qnx_pflags(~0, _PPF_SERVER, NULL, NULL)) /* Get system messages. */
  {
    perror("tmk1553b:man_init:qnx_pflags error");
    exit(-1);
  }

  if ((devno = qnx_device_attach()) == -1) /* Attach a device number to our I/O manager */
  {
    perror("tmk1553b:man_init:qnx_device_attach error");
    exit(-1);
  }

  if (qnx_osinfo(0, &osdata) == -1)       
  {
    perror("tmk1553b:man_init:qnx_osinfo error");
    exit(-1);
  }  

    /* Get a pointer to system time segment. */
  timep = MK_FP(osdata.timesel, offsetof(struct _timesel, seconds));

  if (!(nTmkType == MRTX || nTmkType == MRTXI))
  {
    if (qnx_prefix_attach(devicename, NULL, 0))  /* Attach prefix for tmk */
    {
      perror("tmk1553b:man_init:qnx_prefix_attach error");
      exit(-1);
    }
  }
  if (nTmkType == MRTX || nTmkType == MRTXI)
  {
    for (i = 0; i < mrtgetnrt(); ++i)
    {
      if (mrtgetnrt() <= 10)
        sprintf(devicenamert, "%s_rt%d", devicename, i);
      else
        sprintf(devicenamert, "%s_rt%02d", devicename, i);
      if (qnx_prefix_attach(devicenamert, NULL, i + mrtgetrt0()))  /* Attach prefix for rt */
      {
        perror("tmk1553b:man_init:qnx_prefix_attach error2");
        exit(-1);
      }
    }
  }
  if (!(fd_ctrl = __init_fd(getpid())))
  {
    perror("tmk1553b:man_init:__init_fd error");
    exit(-1);
  }
  
  if ((tmk_proxy = qnx_proxy_attach(0, NULL, 0, -1)) == -1)
  {
    perror("tmk1553b:man_init:qnx_proxy_attach error");
    exit(-1);
  }

  /* Initialize individual stat structure. */
  tmk_stat.st_ino = 1;
  tmk_stat.st_rdev = (dev_t) ((getnid() << 16L) + (devno << 10L)); /* Major and Minor Device number w/node ID. */
  tmk_stat.st_mode = S_IFNAM | S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP | S_IROTH | S_IWOTH;
  tmk_stat.st_nlink = 1;           /* number of links.       */
  tmk_stat.st_ftime = *timep;      /* Time of creation.      */
  tmk_stat.st_atime = *timep;      /* Time of last access.    */
  tmk_stat.st_mtime = *timep;      /* Time of last modification.*/
  tmk_stat.st_ctime = *timep;      /* Time of last status change.  */
  tmk_stat.st_uid = 0;
  tmk_stat.st_gid = 0;
  tmk_stat.st_status = _FILE_USED;
  tmk_stat.st_size = 0;
}
