#include "define.h"
#include <sys/sys_msg.h>
#include <sys/io_msg.h>
#include <sys/stat.h>
#include <stdio.h>
#include <errno.h>
#include "proto.h"
#include "tmkstat.h"

/* These structures may be modified as desired. They are used as examples
   on what you can setup.
   Note that the 'union' described at the end of this file should have the
   first group of structure unions constant. This is the message structure
   that an application will use to talk to the manager.
*/

/*
typedef struct
{
  short nType;
  char szName[10];
  unsigned short wPorts1;
  unsigned short wPorts2;
  unsigned short wIrq1;
  unsigned short wIrq2;
  unsigned short wIODelay;
} TUserTmkConfig;
*/

typedef struct
{
  short nType;
//  char szName[10];
  unsigned short wPorts1;
  unsigned short wPorts2;
//  unsigned short wIrq1;
//  unsigned short wIrq2;
//  unsigned short wIODelay;
  unsigned short wHiddenPorts;
  int nTmk;
//  int fLoaded;
//  int nLoadType;
//  int fIrqShared;
//  int nDev;
//  int nSubDev;
//  int nBusType;
//  int nBusNumber;
//  int hIrq;
  int fLocalReadInt;
  int fMRT;
} TTmkConfig;

typedef struct
{
  int nInt;
  unsigned short wMode;
  unsigned short awEvData[3];
} TListEvD, *pTListEvD;

struct ocb
{
  mode_t      mode;          /* Access modes (ie. O_RDWR). */
  short int   oflags;
  short int   count;         /* number of fd's attached to this OCB. */
//  short int   nbytes;
//  short int   cur_nbytes;
//  char far    *bufp;
  int wTmkNumber;
  unsigned short wModeDeep;
  unsigned short bcBus;
  pid_t       tmkProxy;
  unsigned short wProxyMode;
  volatile unsigned short aiPxD[512];
  unsigned short iiPxD;
  volatile unsigned short ciPxD;
  struct ocb *pNext;
};

/* This union encompasses all the possible message types that the IO manager
   will receive. The replies are also contained in this union.
   The first paragraph of structures (up to and including 'fstat_reply') 
   will ALWAYS be the same for all IO managers.
   You can add other message types in this union if you'd like. In this
   example I've added a extra message type for an incoming proxy.
*/

union msg
{
  short unsigned              type;
  short unsigned              status;
  struct _sysmsg_hdr          sysmsg;
  struct _sysmsg_hdr_reply    sysmsg_reply;
  struct _io_open             open;  /* IO_HANDLE uses this one also.*/
  struct _io_open_reply       open_reply;
  struct _io_dup              dup;   /* Used for fork, exec, dup, dup2()'s.*/
  struct _io_dup_reply        dup_reply;
  struct _io_close            close;
  struct _io_close_reply      close_reply;
  struct _io_read             read;
  struct _io_read_reply       read_reply;
  struct _io_write            write;
  struct _io_write_reply      write_reply;
  struct _io_ioctl            ioctl;
  struct _io_ioctl_reply      ioctl_reply;
  struct _io_utime            utime;
  struct _io_utime_reply      utime_reply;
  struct _io_chmod            chmod;
  struct _io_chmod_reply      chmod_reply;
  struct _io_chown            chown;
  struct _io_chown_reply      chown_reply;
  struct _io_fstat            fstat;
  struct _io_fstat_reply      fstat_reply;
  struct _io_readdir          readdir;
  struct _io_readdir_reply    readdir_reply;
  struct _io_rewinddir        rewinddir;
  struct _io_rewinddir_reply  rewinddir_reply;
   
  /* user defined messages after this point */
  struct _tmk_msg_header        io_tmkll;

  struct __tmk_lock             t_l;
  struct __tmk_unlock           t_ul;
  struct __rtarb_lock           ra_l;
  struct __rtarb_unlock         ra_ul;
  struct __rtansw_lock          raw_l;
  struct __rtansw_unlock        raw_ul;
  struct __tmkdefmode           tdm;
  struct __tmkundefmode         tudm;
  struct __tmkgetmode           tgm;
  struct __bcreset              br;
  struct __rtreset              rr;
  struct __mtreset              mr;
  struct __tmkdefproxy          tdp;
  struct __tmkmakeproxy         tmp;
  struct __tmkgetpxd            tgpd;
  struct __bcgetmaxbase         bgmb;
  struct __bcgetw               bgw;
  struct __bcputw               bpw;
  struct __bcgetblk             bgbl;
  struct __bcputblk             bpbl;
  struct __bcdefbus             bdbs;
  struct __bcgetbus             bgbs;
  struct __bcstart              bst;
  struct __bcstartx             bsx;
  struct __bcgetansw            bgaw;
  struct __bcdeflink            bdl;
  struct __bcgetlink            bgl;
  struct __bcstop               bsp;
  struct __bcgetstate           bgs;
  struct __rtgetmaxpage         rgmp;
  struct __rtdefpage            rdp;
  struct __rtgetpage            rgp;
  struct __rtdefpagepc          rdpp;
  struct __rtgetpagepc          rgpp;
  struct __rtdefpagebus         rdpb;
  struct __rtgetpagebus         rgpb;
  struct __rtdefaddress         rda;
  struct __rtgetaddress         rga;
  struct __rtgetw               rgw;
  struct __rtputw               rpw;
  struct __rtgetblk             rgbl;
  struct __rtputblk             rpbl;
  struct __rtsetanswbits        rsab;
  struct __rtclranswbits        rcab;
  struct __rtgetanswbits        rgab;
  struct __rtgetflags           rgfs;
  struct __rtputflags           rpfs;
  struct __rtsetflag            rsf;
  struct __rtclrflag            rcf;
  struct __rtgetflag            rgf;
  struct __rtgetstate           rgs;
  struct __rtbusy               rb;
  struct __rtlock               rl;
  struct __rtunlock             rul;
  struct __rtgetcmddata         rgcd;
  struct __rtputcmddata         rpcd;
  struct __rtdefmode            rdm;
  struct __rtgetmode            rgm;
  struct __tmksetcwbits         tscb;
  struct __tmkclrcwbits         tccb;
  struct __tmkgetcwbits         tgcb;
  struct __mtgetsw              mgsw;
  struct __bc_def_tldw          b_d_t;
  struct __bc_enable_di         b_e_d;
  struct __bc_disable_di        b_d_d;
  struct __bcdefirqmode         bdim;
  struct __bcgetirqmode         bgim;
  struct __rtdefirqmode         rdim;
  struct __rtgetirqmode         rgim;
  struct __mtdefirqmode         mdim;
  struct __mtgetirqmode         mgim;
  struct __tmkgetinfo           tgi;
  struct __bcdefminbase         bdnb;
  struct __bcgetminbase         bgnb;
  struct __tmkproxymode         tpm;
  struct __tmktimer             tt;
  struct __tmkgettimer          tgt;
  struct __tmkgettimerl         tgtl;
  struct __bcgetmsgtime         bgmt;
  struct __rtgetmsgtime         rgmt;
  struct __tmkgethwver          tghwv;
  struct __tmkGetVersion        tGV;
  struct __rtenable             re;
  struct __mrtgetmaxn           mrgmn;
//#define SS_MRTCONFIG 0x55 ?
//#define SS_MRTSELECTED 0x56 ?
  struct __mrtgetstate          mrgs;
  struct __mrtdefbrcsubaddr0    mrdbsa0;
  struct __mrtreset             mrr;
  struct __tmkgetpxtime         tgpt;
  struct __tmkswtimer           tswt;
  struct __tmkgetswtimer        tgswt;
  struct __tmktimeout           tto;

  struct __tmk_lock_reply       t_l_r;
  struct __tmk_unlock_reply     t_ul_r;
  struct __rtarb_lock_reply     ra_l_r;
  struct __rtarb_unlock_reply   ra_ul_r;
  struct __rtansw_lock_reply    raw_l_r;
  struct __rtansw_unlock_reply  raw_ul_r;
  struct __tmkdefmode_reply     tdm_r;
  struct __tmkundefmode_reply   tudm_r;
  struct __tmkgetmode_reply     tgm_r;
  struct __bcreset_reply        br_r;
  struct __rtreset_reply        rr_r;
  struct __mtreset_reply        mr_r;
  struct __tmkdefproxy_reply    tdp_r;
  struct __tmkmakeproxy_reply   tmp_r;
  struct __tmkgetpxd_reply      tgpd_r;
  struct __bcgetmaxbase_reply   bgmb_r;
  struct __bcgetw_reply         bgw_r;
  struct __bcputw_reply         bpw_r;
  struct __bcgetblk_reply       bgbl_r;
  struct __bcputblk_reply       bpbl_r;
  struct __bcdefbus_reply       bdbs_r;
  struct __bcgetbus_reply       bgbs_r;
  struct __bcstart_reply        bst_r;
  struct __bcstartx_reply       bsx_r;
  struct __bcgetansw_reply      bgaw_r;
  struct __bcdeflink_reply      bdl_r;
  struct __bcgetlink_reply      bgl_r;
  struct __bcstop_reply         bsp_r;
  struct __bcgetstate_reply     bgs_r;
  struct __rtgetmaxpage_reply   rgmp_r;
  struct __rtdefpage_reply      rdp_r;
  struct __rtgetpage_reply      rgp_r;
  struct __rtdefpagepc_reply    rdpp_r;
  struct __rtgetpagepc_reply    rgpp_r;
  struct __rtdefpagebus_reply   rdpb_r;
  struct __rtgetpagebus_reply   rgpb_r;
  struct __rtdefaddress_reply   rda_r;
  struct __rtgetaddress_reply   rga_r;
  struct __rtgetw_reply         rgw_r;
  struct __rtputw_reply         rpw_r;
  struct __rtgetblk_reply       rgbl_r;
  struct __rtputblk_reply       rpbl_r;
  struct __rtsetanswbits_reply  rsab_r;
  struct __rtclranswbits_reply  rcab_r;
  struct __rtgetanswbits_reply  rgab_r;
  struct __rtgetflags_reply     rgfs_r;
  struct __rtputflags_reply     rpfs_r;
  struct __rtsetflag_reply      rsf_r;
  struct __rtclrflag_reply      rcf_r;
  struct __rtgetflag_reply      rgf_r;
  struct __rtgetstate_reply     rgs_r;
  struct __rtbusy_reply         rb_r;
  struct __rtlock_reply         rl_r;
  struct __rtunlock_reply       rul_r;
  struct __rtgetcmddata_reply   rgcd_r;
  struct __rtputcmddata_reply   rpcd_r;
  struct __rtdefmode_reply      rdm_r;
  struct __rtgetmode_reply      rgm_r;
  struct __tmksetcwbits_reply   tscb_r;
  struct __tmkclrcwbits_reply   tccb_r;
  struct __tmkgetcwbits_reply   tgcb_r;
  struct __mtgetsw_reply        mgsw_r;
  struct __bc_def_tldw_reply    b_d_t_r;
  struct __bc_enable_di_reply   b_e_d_r;
  struct __bc_disable_di_reply  b_d_d_r;
  struct __bcdefirqmode_reply   bdim_r;
  struct __bcgetirqmode_reply   bgim_r;
  struct __rtdefirqmode_reply   rdim_r;
  struct __rtgetirqmode_reply   rgim_r;
  struct __tmkgetinfo_reply     tgi_r;
  struct __bcdefminbase_reply   bdnb_r;
  struct __bcgetminbase_reply   bgnb_r;
  struct __tmkproxymode_reply   tpm_r;
  struct __tmktimer_reply       tt_r;
  struct __tmkgettimer_reply    tgt_r;
  struct __tmkgettimerl_reply   tgtl_r;
  struct __bcgetmsgtime_reply   bgmt_r;
  struct __rtgetmsgtime_reply   rgmt_r;
  struct __tmkgethwver_reply    tghwv_r;
  struct __tmkGetVersion_reply  tGV_r;
  struct __rtenable_reply       re_r;
  struct __mrtgetmaxn_reply     mrgmn_r;
//#define SS_MRTCONFIG 0x55 ?
//#define SS_MRTSELECTED 0x56 ?
  struct __mrtgetstate_reply    mrgs_r;
  struct __mrtdefbcrsubaadr0_reply mrdbsa0_r;
  struct __mrtreset_reply       mrr_r;
  struct __tmkgetpxtime_reply   tgpt_r;
  struct __tmkswtimer_reply     tswt_r;
  struct __tmkgetswtimer_reply  tgswt_r;
  struct __tmktimeout_reply     tto_r;
};
