#include <string.h>
#include <fcntl.h>
#include <sys/psinfo.h>
#include <sys/fd.h>
#include "define.h"
#include "struct.h"
#include "extern.h"

int map_ocb(int pid, int fd, struct ocb *ocb)
{
  /* Use unit as index into ocb array. */
  if (qnx_fd_attach(pid, fd, 0, 0, 0, 0, (unsigned)ocb) == -1)
  {
    io_msg.status = errno;
    return (-1);
  }
  return (0);
}

struct ocb * get_ocb(pid_t pid, int fd)
{
  struct ocb *ocb;
   
  ocb = (struct ocb *)__get_fd(pid, fd, fd_ctrl);
  if (ocb == (struct ocb *)0 || ocb == (struct ocb *)-1)
  {
    io_msg.status = EBADF;
    return((struct ocb *)-1);
  }
  return (ocb);
}

mode_t check_access(pid_t pid, struct stat *sptr)
{
  mode_t mode;
  uid_t uid;
  gid_t gid;

  mode_t perm_mode[3] = { S_IREAD, S_IWRITE, S_IREAD|S_IWRITE };

          /* Get egid and euid of process pid.*/
  get_ids(pid, &uid, &gid);
          /* Make sure the calling process has correct permissions. */
  mode = perm_mode[io_msg.open.oflag & O_ACCMODE];
  mode &= (uid == 0) ?
          S_IRWXU : (uid == sptr->st_uid) ? (sptr->st_mode & S_IRWXU) :
          (gid == sptr->st_gid) ? (sptr->st_mode & S_IRWXG) << 3 :
          (sptr->st_mode & S_IRWXO) << 6;
  return (mode);
}

void get_ids(pid_t pid, uid_t *uid, gid_t *gid)
{
  struct _psinfo3 psinfo3;
  
  __get_pid_info(pid, &psinfo3, fd_ctrl);
  *uid = psinfo3.euid;
  *gid = psinfo3.egid;
}
